<?php
declare(strict_types=1);

namespace Flat101\EnhancedEcommerce\Model\DataLayer\Generator;

use Flat101\EnhancedEcommerce\Model\DataLayer\DataProvider\AdapterDataProvider;
use Flat101\EnhancedEcommerce\Model\DataLayer\Dto;
use Flat101\EnhancedEcommerce\Model\Handler;
use Flat101\EnhancedEcommerce\Model\UnsetProperty;
use Flat101\EnhancedEcommerce\Model\Session;
use Flat101\EnhancedEcommerce\Helper\Config;
use Magento\Catalog\Api\CategoryRepositoryInterface;
use Magento\Catalog\Model\Product as ProductEntity;
use Magento\Framework\Exception\NoSuchEntityException;
use Magento\Quote\Model\Quote\Item as QuoteItem;
use Magento\Sales\Model\Order\Item as OrderItem;
use Psr\Log\LoggerInterface;

class Product
{
    use UnsetProperty;

    /** @var Handler\ProductHandler */
    private $productHandler;

    /** @var Handler\ItemHandler */
    private $itemHandler;

    /** @var CategoryRepositoryInterface */
    private $categoryRepository;

    /** @var AdapterDataProvider */
    private $adapterDataProvider;

    /** @var Config */
    private $config;

    /** @var Session */
    private $session;

    /** @var LoggerInterface */
    private $logger;

    public function __construct(
        Handler\ProductHandler $productHandler,
        Handler\ItemHandler $itemHandler,
        CategoryRepositoryInterface $categoryRepository,
        AdapterDataProvider $adapterDataProvider,
        Config $config,
        Session $session,
        LoggerInterface $logger
    ) {
        $this->productHandler = $productHandler;
        $this->categoryRepository = $categoryRepository;
        $this->itemHandler = $itemHandler;
        $this->adapterDataProvider = $adapterDataProvider;
        $this->config = $config;
        $this->session = $session;
        $this->logger = $logger;
    }

    /**
     * @param ProductEntity|null $entity
     * @param null|int $quantity
     * @param null|OrderItem|QuoteItem $item
     * @param null $additionalInfo
     * @return null | Dto\Product
     */
    public function generate(?ProductEntity $entity, $quantity = null, $item = null, $additionalInfo = null): ?Dto\Product
    {
        if (null !== $entity) {
            $this->productHandler->setProduct($entity);
        }

        /** @var ProductEntity $product */
        $product = $this->productHandler->getProduct();

        if (is_null($product)) {
            return null;
        }

        if (is_null($additionalInfo)) {
            $allAdditionalInfo = $this->getAdditionalProductInfo($product);
            $additionalInfo = ($allAdditionalInfo[$product->getId()]) ?? null;
        }

        $category = $product->getCategory();
        if (null === $category) {
            $categoryIds = $product->getCategoryIds();

            try {
                $category = $this->categoryRepository->get(\reset($categoryIds));
            } catch (NoSuchEntityException $exception) {
                $this->logger->notice($exception->getMessage());
            }
        }

        if (null !== $category) {
            $this->productHandler->setCategory($category);
        }

        $productDto = new Dto\Product();
        $productDto->id = $product->getData($this->productHandler->productIdentifier());
        $productDto->name = $product->getName();
        $productDto->price = $this->getPrice($product);
        $productDto->category = ($additionalInfo['category']) ?? $this->productHandler->getCategoryPath();
        $productDto->brand = $this->productHandler->getBrandValue();
        $productDto->position = ($additionalInfo['position']) ?? 0;

        if (null !== $quantity) {
            $productDto->quantity = $quantity;
        }

        if (!is_null($variantValue = $this->checkVariantValue($item))) {
            $productDto->variant = $variantValue;
        }

        if (null === $variantValue) {
            $this->unset($productDto, ['variant']);
        }
        if (null === $item) {
            $this->unset($productDto, ['quantity']);
        }

        $this->adapterDataProvider->handle($productDto, [
            'category' => $category,
            'product' => $product,
        ]);

        return $productDto;
    }

    private function getPrice(ProductEntity $product): float
    {
        return floatval($product->getSpecialPrice()) ?: floatval($product->getFinalPrice());
    }

    private function checkVariantValue($item): ?string
    {
        return $this->productHandler->checkVariantValue($item);
    }

    protected function getAdditionalProductInfo($product): array
    {
        $response = $this->session->getProductAdditionalInfo(true);
        $allImpressionProducts = $this->session->getImpressionCatalogProducts(true);

        foreach ($allImpressionProducts as $impressionProduct) {
            if ($impressionProduct->id == $product->getId()) {
                $list = explode($this->config->getCategorySeparator(), $impressionProduct->category);
                $response[$impressionProduct->id] = [
                    'list'      => trim(end($list)),
                    'category'  => $impressionProduct->category,
                    'position'  => $impressionProduct->position
                ];
            }
        }

        $this->session->setProductAdditionalInfo($response);
        return $response;
    }
}
